var fs = require('fs');
var _ = require('underscore');

exports.create = function(app) {
  var api = [];
  versions = fs.readdirSync(__dirname + '/api');
  for (var i in versions) {
    var version = versions[i];
    api[version] = readFiles(__dirname + '/api/' + version, app);
  }
  createRoutes(api, app, '/api');
  return api;
};

function readFiles(dir, app) {
  var api = [];
  var files = fs.readdirSync(dir);
  for (var i in files) {
    var file = files[i];
    var stat = fs.statSync(dir + '/' + file);
    if (stat.isDirectory()) {
      api[file] = readFiles(dir + '/' + file, app);
    } else {
      api[file] = require(dir + '/' + file);
    }
  }
  return api;
}

function createRoutes(api, app, url) {
  console.log(api);
  for (var route in api) {
    var obj = api[route];
    var endpoint = route.replace('.js', '');
    var routeUrl = url + '/' + endpoint;
    if (_.isArray(obj)) {
      createRoutes(obj, app, url + '/' + route);
      continue;
    }
    if (obj.findAll) {
      console.log(routeUrl);
      app.get(routeUrl, obj.findAll);
    }
    if (obj.findById) {
      app.get(routeUrl + '/:id', obj.findById);
    }
    if (obj.create) {
      app.post(routeUrl, obj.create);
    }
    if (obj.update) {
      app.put(routeUrl + '/:id', obj.update);
    }
    if (obj.destroy) {
      app.delete(routeUrl + '/:id', obj.destroy);
    }
    if (obj.routes) {
      for (var i in obj.routes) {
        if (obj.routes[i].type == 'post') {
          app.post(routeUrl + i, obj.routes[i].handler);
        } else if (obj.routes[i].type == 'get') {
          app.get(routeUrl + i, obj.routes[i].handler);
        } else if (obj.routes[i].type == 'put') {
          app.put(routeUrl + i, obj.routes[i].handler);
        } if (obj.routes[i].type == 'delete') {
          app.delete(routeUrl + i, obj.routes[i].handler);
        }
      }
    }
  }
}
