var q = require('q');
var serialosc = require('../../../serialosc');

exports.findById = function (req, res) {
  var id = parseInt(req.params.id, 10);
  var device = serialosc.device(id);
  if (!device) {
    return error(res, 404, 'Device not found');
  }
  res.json(device);
};

exports.findAll = function (req, res) {
  res.json(serialosc.devices());
};

exports.update = function (req, res) {
  var deferred = q.defer();
  var id = parseInt(req.params.id, 10);
  if (!serialosc.device(id)) {
    return error(res, 404, 'Device not found');
  }
  if (req.body.focus) {
    var focus = parseInt(req.body.focus, 10);
    if (focus) {
      serialosc.focus(id)
      .then(function(device) {
        res.json(device);
      });
      return;
    } else {
      device.unfocus();
    }
  }
  if (req.body.rotation) {
    var rotation = parseInt(req.body.rotation, 10);
    if (!_.contains([0, 90, 180, 270], rotation)) {
      return error(res, 404, 'Rotation not 0, 90, 180, or 270');
    }
    serialosc.setRotation(id, req.body.rotation);
  }
  if (req.body.prefix) {
    serialosc.setPrefix(id, req.body.prefix);
  }
  res.json(device);
};

function error(res, code, msg) {
  res.statusCode = code;
  res.end(msg);
  return;
}