var fs = require('fs');
var _ = require('underscore');

var nextBlockId = 0;

var blocks = {
  modules: [],
  running: []
};

exports.modules = {
  findAll: function() {
    refreshBlockList();
    return blocks.modules;
  },
  findById: function (id) {
    refreshBlockList();
    var block = _.where(blocks.modules, {id: id}).pop();
    return block || false;
  }
};

var validKeys = ['sizeX', 'sizeY', 'offsetX', 'offsetY', 'deviceId'];
exports.running = {
  findAll: function() {
    return blocks.running;
  },
  findAllByDeviceId: function(deviceId) {
    var blocks = _.where(blocks.running, {deviceId: deviceId});
  },
  findById: function (id) {
    var block = _.where(blocks.running, {id: id}).pop();
    return block || false;
  },
  create: function(body) {
    refreshBlockList();
    var block = _.where(blocks.modules, {id: body.id}).pop();
    if (!block) {
      return false;
    }
    var newBlock = _.extend({}, block);
    newBlock.id = ++nextBlockId;
    _.each(validKeys, function(key) {
      if (body[key]) newBlock.options[key] = body[key];
    });
    blocks.running.push(newBlock);
    return newBlock;
  },
  update: function(id, body) {
    var block = _.where(blocks.running, {id: id}).pop();
    if (!block) {
      return false;
    }
    _.each(validKeys, function(key) {
      if (body[key]) block.options[key] = body[key];
    });
    return block;
  }
};

function refreshBlockList() {
  blocks.modules = [];
  var files = fs.readdirSync(__dirname + '/blocks');
  var nextId = 0;
  for (var i in files) {
    var name = files[i].match(/(.*)\.js/);
    if (!name[1]) continue;
    var path = __dirname + '/blocks/' + files[i];
    var block = require(path);
    block.id = ++nextId;
    blocks.modules.push(block);

    // blocks.modules.push({
    //   id: ++nextId,
    //   options: block.options
    // });
  }
}