var _ = require('underscore');
//var midigrid = require('midigrid');
var midigrid = require('/Users/mac/src/node-midigrid/lib/midigrid');

var socket = require('./socket');

var nextId = 0;

var emulators = {
  modules: [
    {
      id: 1,
      name: 'launchpad',
      serialoscId: 'emulator',
      prefix: '/monome',
      serialoscMapFunc: function(noteNum) {
        var x = noteNum % 8;
        var y = Math.floor(noteNum / 8) / 2;
        console.log('press event: converted note ' + noteNum + ' to ' + x + ', ' + y);
        return [x, y];
      },
      midiMapFunc: function(data) {
        var noteNum = (data.y * 16) + data.x;
        console.log('led event: converted ' + data.x + ', ' + data.y + ' to note ' + noteNum);
        return noteNum;
      },
      velocityOn: function(data) {
        return 127;
      },
      velocityOff: function(data) {
        return 0;
      }
    }
  ],
  running: []
};

exports.modules = {
  findAll: function () {
    return emulators.modules;
  },
  findById: function (id) {
    var module = _.where(emulators.modules, {id: id}).pop();
    return module || false;
  }
};

exports.running = {
  findAll: function () {
    return emulators.running;
  },
  findById: function (id) {
    var emulator = _.where(emulators.running, {id: id}).pop();
    return emulator || false;
  },
  create: function(moduleId, body) {
    var module = _.where(emulators.modules, {id: moduleId}).pop();
    if (!module) {
      return false;
    }
    var newEmulator = _.extend({}, module, body);
    newEmulator.id = ++nextId;
    newEmulator.serialoscId += "" + nextId;
    newEmulator.device = midigrid.create(newEmulator);
    // TODO: add chaining for .start()
    newEmulator.device.start();
    emulators.running.push(newEmulator);
    socket.broadcast('updateRunningEmulators');
    return newEmulator;
  },
  destroy: function(id) {
    var emulator = _.where(emulators.running, {id: id}).pop();
    if (!emulator) {
      return false;
    }
    emulator.device.stop();
    emulators.running = _.without(emulators.running, emulator);
    socket.broadcast('updateRunningEmulators');
    return emulator;
  }
};
