var sys = require('sys');
var q = require('q');
var _ = require('underscore');
var monome = require('monome');

var devices = [];

monome.on('deviceFound', function (device) {
  devices.push(device);
});
monome.on('deviceLost', function (device) {
  devices = _.without(devices, device);
});

exports.devices = function() {
  return devices;
};

exports.device = function(id) {
  return _.where(devices, {id: id}).pop();
};

exports.focus = function(id) {
  var deferred = q.defer();
  var device = exports.device(id);
  device.focus(function() {
    device.on('press', function (x, y, s) {
      console.log("[ press] " + x + ", " + y + ", " + s);
      device.oscOut("/grid/led/set", x, y, s);
    });
    device.on('oscOut', function (msg) {
      console.log("[oscOut] " + sys.inspect(msg));
    });
    device.on('oscIn', function (msg) {
      console.log("[ oscIn] " + sys.inspect(msg));
    });
    deferred.resolve(device);
  });
  return deferred.promise;
};

exports.unfocus = function(id) {
  var device = exports.device(id);
  device.unfocus();
};

exports.setPrefix = function(id, prefix) {
  var device = exports.device(id);
  device.prefix = prefix;
  device.oscOut('/sys/prefix', prefix);
};

exports.setRotation = function(id, rotation) {
  device.rotation = rotation;
  device.oscOut('/sys/rotation', rotation);
};