var devices = [];
var activeDevice = false;
var midiIns = [];
var midiOuts = [];
var emulatorModules = [];
var emulators = [];
var blockModules = [];
var blocks = [];
var socket;

$(document).ready(function() {
  var socket = io.connect('http://localhost:8001');
  socket.on('updateRunningEmulators', function() {
    console.log('get running emulators');
    getRunningEmulators();
  });

  socket.on('updateDevices', function() {
    console.log('get devices');
    getDevices()
    .then(function(devicesResponse) {
      devices = devicesResponse;
      redrawDeviceButtons();
    });
  });

  $.when(
    getDevices(),
    getRunningBlocks(),
    getRunningEmulators()
  )
  .then(function(devicesResponse, blocksResponse, emulatorsResponse) {
    devices = devicesResponse[0];
    blocks = blocksResponse[0];
    emulators = emulatorsResponse[0];
    redrawDeviceButtons();
    console.log(devices);
    console.log(blocks);
    console.log(emulators);
  });

  $('button').button();

  $('#add-emulator').on('click', function() {
    setupAddEmulatorDialog(function() {
      $('#add-emulator-dialog').dialog({
        buttons: {
          add: function() {
            createEmulator();
          },
          cancel: function() {
            $(this).dialog('close');
          }
        }
      });
    });
  });

  $('#add-block').on('click', function() {
    setupAddBlockDialog(function() {
      $('#add-block-dialog').dialog({
        buttons: {
          add: function() {
            createBlock();
          },
          cancel: function() {
            $(this).dialog('close');
          }
        }
      });
    });
  });
  // var device = makeGrid(8, 8, 'device');
  // $('.device-zone').append(device);
  // device.attr('id', 'device');

  // var block = addBlock('#block');
  // block.find('.block-cell').css('backgroundColor', '#0ac');
  // $('.block-zone').append(block);

  // var block2 = addBlock('#block2');
  // block2.find('.block-cell').css('backgroundColor', '#af0');
  // $('.block-zone').append(block2);

});

var getRunningEmulators = function() {
  return $.ajax({
    url: '/api/v1/emulators/running'
  });
};

var getDevices = function() {
  return $.ajax({
    url: '/api/v1/serialosc/devices'
  });
};

var getRunningBlocks = function() {
  return $.ajax({
    url: '/api/v1/blocks/running'
  });
};

var redrawDeviceButtons = function() {
  $('#devices').html('');
  $.each(devices, function(i, device) {
    var btn = $('<button></button>');
    btn.text(device.service.name);
    btn.on('click', function(e) {
      devices[i] = device;
      if (!device.focused) {
        focusDevice(device, function(device) {
          showDevice(device);
        });
      } else {
        showDevice(device);
      }
    });
    $('#devices').append(btn);
    btn.button();
  });
  $('.device-grid').each(function() {
    var deviceId = $(this).attr('data-device-id');
    var found = false;
    for (var i = 0; i < devices.length; i++) {
      if (devices[i].id == deviceId) {
        found = true;
      }
    }
    if (!found) {
      $(this).remove();
    }
  });
};

var showDevice = function(device) {
  activeDevice = device;
  if (!device.grid) {
    device.grid = makeDeviceGrid(device);
    $('.device-zone').append(device.grid);
  }
  $('.show-hide').hide();
  $('#device-' + device.id).show();
};

var focusDevice = function(device, cb) {
  $.ajax({
    url: '/api/v1/serialosc/devices/' + device.id,
    type: 'put',
    data: {
      focus: 1
    },
    dataType: 'json',
    success: function(response) {
      cb(response);
    }
  });
};

var setupAddEmulatorDialog = function(cb) {
  $.when(
    $.ajax({url: '/api/v1/emulators/modules'}),
    $.ajax({url: '/api/v1/midi/ins'}),
    $.ajax({url: '/api/v1/midi/outs'})
  )
  .then(
    function(emulators, midiIn, midiOut) {
      emulatorModules = emulators[0];
      midiIns = midiIn[0];
      midiOuts = midiOut[0];

      $.each(emulatorModules, function(i, module) {
        var option = $('<option></option>');
        option.val(module.id);
        option.text(module.name);
        $('#emulator-module').append(option);
      });

      $.each(midiIns, function(i, midiIn) {
        var option = $('<option></option>');
        option.val(midiIn.name);
        option.text(midiIn.name);
        $('#emulator-midi-in').append(option);
      });

      $.each(midiOuts, function(i, midiOut) {
        var option = $('<option></option>');
        option.val(midiOut.name);
        option.text(midiOut.name);
        $('#emulator-midi-out').append(option);
      });

      cb();
    }
  );
};

var setupAddBlockDialog = function(cb) {
  $('#block-module').html('');
  $.ajax({url: '/api/v1/blocks/modules'})
  .then(
    function(blocks) {
      $('#block-module').off('change');
      $('#block-module').on('change', function(e) {
        var id = $(this).val();
        console.log('change');
        $.each(blocks, function(i, module) {
          if (module.id == id) {
            setupBlockSizeDropdown(module);
          }
        });
      });
      $.each(blocks, function(i, module) {
        var option = $('<option></option>');
        option.val(module.id);
        option.text(module.name);
        $('#block-module').append(option);
      });

      cb();
      $('#block-module').trigger('change');
    }
  );
};

var setupBlockSizeDropdown = function(block) {
  console.log(block);
  var opt;
  $('#block-size-x').html('');
  for (var x = block.options.minSizeX; x <= block.options.maxSizeX; x++) {
    opt = $('<option></option>');
    opt.val(x);
    opt.text(x);
    $('#block-size-x').append(opt);
  }
  $('#block-size-y').html('');
  for (var y = block.options.minSizeY; y <= block.options.maxSizeY; y++) {
    opt = $('<option></option>');
    opt.val(y);
    opt.text(y);
    console.log(opt);
    $('#block-size-y').append(opt);
  }
};

var createEmulator = function() {
  $.ajax({
    url: '/api/v1/emulators/running',
    type: 'post',
    data: {
      moduleId: $('#emulator-module').val(),
      name: $('#emulator-name').val(),
      prefix: $('#emulator-prefix').val(),
      midiIn: $('#emulator-midi-in').val(),
      midiOut: $('#emulator-midi-out').val()
    },
    success: function() {
      $('#add-emulator-dialog').dialog('close');
    }
  });
};

var getEmulator = function(device) {
  for (var i = 0; i < emulators.length; i++) {
    if (emulators[i].serialoscId == device.serialoscId) {
      return emulators[i];
    }
  }
  return false;
};

var refreshBlockPanel = function(block) {
  console.log(block);
};

var createBlock = function() {
  var moduleId = $('#block-module').val();
  var sizeX = $('#block-size-x').val();
  var sizeY = $('#block-size-y').val();
  $.ajax({
    url: '/api/v1/blocks/running',
    type: 'post',
    data: {
      id: moduleId,
      deviceId: activeDevice.id,
      sizeX: sizeX,
      sizeY: sizeY
    }
  })
  .then(function(block) {
    var blockElem = makeBlockElem(block);
    $('.block-zone').prepend(blockElem);
    makeBlock(blockElem);
    block.elem = blockElem;
    blocks.push(block);
  });
};

var makeBlockElem = function(block) {
  var blockElem = $('<div></div>');
  blockElem.id = 'block-' + block.id;
  blockElem.data('id', block.id);
  blockElem.addClass('block-grid');
  for (var y = 0; y < block.options.sizeY; y++) {
    var row = $('<div></div>');
    row.addClass('block-row');
    blockElem.append(row);
    for (var x = 0; x < block.options.sizeX; x++) {
      var cell = $('<div></div>');
      cell.addClass('block-cell');
      row.append(cell);
    }
  }
  return blockElem;
};

var makeBlock = function(selector) {
  var block = $(selector);
  block.data('x', 0);
  block.data('y', 0);
  block.on('mousedown mouseup', function() {
    $('.block-grid').css('zIndex', '1');
    $(this).css('zIndex', 2);
    refreshBlockPanel(getBlockById($(this).data('id')));
    return true;
  });
  block.draggable({
    snap: '.dropzone',
    snapMode: 'inner',
    stop: function(event, ui) {
      var draggable = $(this).data("ui-draggable");
      block.data('x', 0);
      block.data('y', 0);
      var done = false;
      $.each(draggable.snapElements, function(i, e) {
        if (e.snapping) {
          if (done) return;
          $('.dropzone').each(function() {
            var offset = block.offset();
            var dropOffset = $(this).offset();
            var device = $(this).closest('.device-grid');
            dropOffset.top = Math.floor(dropOffset.top / 10);
            dropOffset.left = Math.floor(dropOffset.left / 10);
            offset.top = Math.floor(offset.top / 10);
            offset.left = Math.floor(offset.left / 10);
            // console.log(offset);
            // console.log(dropOffset.top + ' == ' + offset.top + ' && ' + dropOffset.left + ' == ' + offset.left);
            if (dropOffset.top == offset.top && dropOffset.left == offset.left) {
              var snapX = $(this).data('x');
              var snapY = $(this).data('y');
              block.data('x', $(this).data('x'));
              block.data('y', $(this).data('y'));
              draggable._trigger('placed', event, $.extend({}, ui, {
                device: device,
                snapX: snapX,
                snapY: snapY
              }));
              done = true;
            }
          });
        }
      });
    },
    placed: function(event, ui) {
      console.log('PLACED');
      $.ajax({
        url: '/api/v1/blocks/running/' + block.data('id'),
        type: 'put',
        data: {
          offsetX: ui.snapX,
          offsetY: ui.snapY
        }
      })
      .then(function(result) {
        var blockObj = getBlockById(block.data('id'));
        blockObj.options.offsetX = ui.snapX;
        blockObj.options.offsetY = ui.snapY;
        refreshBlockPanel(blockObj);
      });
    }
  });
  return block;
};

var makeDeviceGrid = function(device) {
  console.log('---');
  console.log(device);
  var grid = $('<div></div>');
  grid.attr('id', 'device-' + device.id);
  grid.addClass('device-grid');
  grid.addClass('show-hide');
  grid.css('display', 'none');
  grid.attr('data-device-id', device.id);
  grid.append('<h2>' + device.service.name + '</h2>');
  var emulator = getEmulator(device);
  if (emulator) {
    var btn = $('<button></button>');
    btn.attr('id', 'stop-emulator-' + emulator.id);
    btn.text('stop emulator');
    btn.css('marginBottom', '25px');
    grid.append(btn);
    btn.button();
    btn.on('click', function(e) {
      $.ajax({
        url: '/api/v1/emulators/running/' + emulator.id,
        type: 'delete',
        success: function() {

        }
      });
    });
  }
  for (var y = 0; y < device.sizeY; y++) {
    var row = $('<div></div>');
    row.addClass('device-row');
    grid.append(row);
    for (var x = 0; x < device.sizeX; x++) {
      var cell = $('<div></div>');
      cell.addClass('device-cell');
      var dropzone = $('<div></div>');
      dropzone.addClass('dropzone');
      cell.append(dropzone);
      dropzone.attr('data-x', x);
      dropzone.attr('data-y', y);
      row.append(cell);
    }
  }
  return grid;
};

var makeGrid = function(device, type) {
  var grid = $('<div class="' + type + '-grid"></div>');
  grid.addClass('device-' + device.id);
  grid.addClass('show-hide');
  grid.css('display', 'none');
  if (type == 'device') {
    grid.attr('id', 'device-' + device.id);
  }
  for (var y = 0; y < device.sizeY; y++) {
    var row = $('<div class="' + type + '-row"></div>');
    grid.append(row);
    for (var x = 0; x < device.sizeX; x++) {
      var cell = $('<div class="' + type + '-cell"></div>');
      if (type == 'device') {
        var dropzone = $('<div class="dropzone"></div>');
        cell.append(dropzone);
        dropzone.attr('data-x', x);
        dropzone.attr('data-y', y);
      }
      row.append(cell);
    }
  }
  return grid;
};

var getBlockById = function(id) {
  var foundBlock = false;
  $.each(blocks, function(i, block) {
    if (block.id == id) {
      foundBlock = block;
    }
  });
  return foundBlock;
};